from scapy.all import (IP_PROTOS, BitEnumField, BitField,
                       ByteEnumField, ByteField, DNSQRField, DNSRRField,
                       FlagsField, IntField, Packet, ShortEnumField,
                       StrField, TCPOptionsField, XByteField)
from scapy.layers.dns import DNSStrField, InheritOriginDNSStrPacket, dnstypes
from scapy.layers.inet6 import ipv6nh


class CustomIP(Packet):
    name = "IP"
    fields_desc = [
        BitField("version", 4, 4),
        FlagsField("flags", 0, 4, ["R", "DF", "MF"]), # normally 3 bits last bit will always be 0
        XByteField("tos", 0),
        ByteField("ttl", 64),
        ByteEnumField("proto", 0, IP_PROTOS),
        ]

class CustomIPv6(Packet):
    name = "IPv6"
    fields_desc = [
        BitField("version", 6, 8), # normally 4 bits, last 4bits will always be 0
        BitField("tc", 0, 8),
        ByteEnumField("nh", 59, ipv6nh),
        ByteField("hlim", 64)
    ]

class CustomTCP(Packet):
    name = "TCP"
    fields_desc = [
        FlagsField("flags", 0x2, 16, "FSRPAUECN"),
        TCPOptionsField("options", "")
    ]

class CustomUDP(Packet):
    name = "UDP"
    fields_desc = []

class CustomHTTP(Packet):
    def self_build(self):
        p = b""

        for f in self.fields_desc:
            val = self.getfieldval(f.name)
            #when Value is not set
            if not val:
                continue
            if f.name not in ['Method', 'Path', 'Status_Code']:
                val = bytes((f.name).encode()) + b": " + bytes(val)
            if f.name in ['Method', 'Path', 'Status_Code']:
                seperator = b' '
            else:
                seperator = b'\r\n'

            p = f.addfield(self, p, val + seperator)
        
        return p


class CustomHTTP_Request(CustomHTTP):
    name = "HTTP Request"
    fields_desc = [
        StrField("Method", "GET"),
        StrField("Path", "/"),
        StrField("User_Agent", None),
        StrField("Content_Type", None),
        StrField("Connection", None),
        StrField("Accept", None),
        StrField("Accept_Charset", None),
        StrField("Accept_Encoding", None),
        StrField("Cookie", None),
        StrField("TE", None)
    ]

class CustomHTTP_Response(CustomHTTP):
    name = "HTTP Response"
    fields_desc = [
        StrField("Status_Code", "200"),
        StrField("Connection", None),
        StrField("Content_Encoding", None),
        StrField("Content_Type", None),
        StrField("Server", None),
        StrField("Set_Cookie", None),
        StrField("Transfer_Encoding", None)
    ]

class CustomDNSQR(InheritOriginDNSStrPacket):
    name = "DNS Question Record"
    show_indent = 0 
    fields_desc = [
        DNSStrField("qname", "none"),
        ShortEnumField("qtype", 1, dnstypes)
    ]

class CustomDNSRR(InheritOriginDNSStrPacket):
    name = "DNS Resource Record"
    show_indent = 0
    fields_desc = [
        DNSStrField("rrname", ""),
        ShortEnumField("type", 1, dnstypes),
        IntField("ttl", 0)
    ]

class CustomDNS(Packet):
    name = "DNS"
    fields_desc = [
        BitField("qr", 0, 1),
        BitEnumField("opcode", 0, 4, {0: "QUERY", 1: "IQUERY", 2: "STATUS"}),
        BitField("aa", 0, 1),
        BitField("tc", 0, 1),
        BitField("rd", 1, 1),
        BitField("ra", 0, 1),
        BitField("z", 0, 1),
        BitField("ad", 0, 1),
        BitField("cd", 0, 1),
        BitEnumField("rcode", 0, 4, {0: "ok", 1: "format-error",
                                     2: "server-failure", 3: "name-error",
                                     4: "not-implemented", 5: "refused"}),
        DNSQRField("qd", "", None),
        DNSRRField("an", "", None),
        DNSRRField("ns", "", None),
        DNSRRField("ar", "", None),
    ]



from scapy.all import Packet
from scapy.layers.dns import DNS

from heifip.layers.transport import TransportPacket
from heifip.plugins.header import CustomDNS, CustomDNSQR, CustomDNSRR


class DNSPacket(TransportPacket):
    def __init__(self, packet: Packet, address_mapping={}, layer_map={}) -> None:
        TransportPacket.__init__(self, packet, address_mapping, layer_map)
        # if DNS in self.layer_map:
        #     self.hash = hashlib.md5(f"{self.packet[DNS].qr}".encode('utf-8')).hexdigest()
    
    def header_preprocessing(self):
        # TODO: Fix issue with DNS processing
        if self.packet[DNS].qd:
            self.__header_preprocessing_message_type(self.packet, "qd")
        if self.packet[DNS].an:
            self.__header_preprocessing_message_type(self.packet, "an")
        if self.packet[DNS].ns:
            self.__header_preprocessing_message_type(self.packet, "ns")
        if self.packet[DNS].ar:
            self.__header_preprocessing_message_type(self.packet, "ar")

        layer_copy = self.packet[DNS]

        new_layer = CustomDNS(
            qr=layer_copy.qr,
            opcode=layer_copy.opcode,
            aa=layer_copy.aa,
            tc=layer_copy.tc,
            rd=layer_copy.rd,
            ra=layer_copy.ra,
            z=layer_copy.z,
            ad=layer_copy.ad,
            cd=layer_copy.cd,
            rcode=layer_copy.rcode,
            qd=layer_copy.qd,
            an=layer_copy.an,
            ns=layer_copy.ns,
            ar=layer_copy.ar,
        )

        self.packet[DNS] /= new_layer

        super().header_preprocessing()

    
    def __header_preprocessing_message_type(self, packet: Packet, message_type: str):
        message = getattr(packet[DNS], message_type)
        if message_type == "qd":
            new_message = CustomDNSQR(qname=message.qname, qtype=message.qtype)
            
            while message:=message.payload:
                new_message /= CustomDNSQR(
                    qname=message.qname,
                    qtype=message.qtype,
                )
        else:
            new_message = CustomDNSRR(
                rrname=message.rrname, type=message.type
            )

            while message:=message.payload:
                new_message /= CustomDNSRR(
                    rrname=message.rrname, type=message.type
                )
        
        setattr(packet[DNS], message_type, new_message)



import binascii

import numpy as np
from scapy.all import Packet, raw

from heifip.images import NetworkTrafficImage


class FlowImageTiledFixed(NetworkTrafficImage):
    def __init__(
        self,
        packets,
        dim=16,
        fill=0,
        cols=3,
    ) -> None:
        NetworkTrafficImage.__init__(self, fill, dim)
        self.packets = packets
        self.cols = cols
        self.matrix, self.binaries = self.__get_matrix_tiled(self.fill, self.dim, self.cols, packets)
        del packets

    def __get_matrix_tiled(self, fill: int, dim: int, cols: int, packets: [Packet]):
        """
            Creates a matrix of a list of Scapy Packet.
            Packets are tiled into a quadratic representation.
        """
        binaries = []
        for packet in self.packets:
            # get Hex data
            hexst = binascii.hexlify(raw(packet.packet))
            # Append octet as integer
            binaries.append(
                [int(hexst[i : i + 2], 16) for i in range(0, len(hexst), 2)]
            )

        result = []
        for x in binaries:
            x = x[: dim * dim]
            x = np.array(x + [fill] * (dim * dim - len(x)))
            x = x.reshape(dim, dim)
            result.append(x)

        # Create tiled image
        fh = self.__tile_images(result, cols, dim)
        # Convert to int
        fh = np.uint8(fh)
        return fh, binaries

    def __tile_images(self, images, cols: int, dim: int):
        """Tile images of same size to grid with given number of columns.

        Args:
            images (collection of ndarrays)
            cols (int): number of colums

        Returns:
            ndarray: stitched image
        """
        k = 0
        rows = []
        for i in range(0, cols):
            row = None
            for j in range(0, cols):
                if len(images) > k:
                    im = images[k]
                else:
                    im = np.zeros((dim, dim))
                
                if row is None:
                    row = im
                else:
                    row = np.concatenate((row, im), axis=1)
                k += 1
                
            rows.append(row)
        tiled = np.concatenate(rows)

        return tiled


from scapy.all import Packet

from heifip.layers.transport import TransportPacket


class SSHPacketProcessor(TransportPacket):
    def __init__(self, packet: Packet, address_mapping={}, layer_map={}):
        TransportPacket.__init__(self, packet, address_mapping, layer_map)

    def header_preprocessing(self):
        super().header_preprocessing()


import hashlib
from typing import Type

from scapy.all import Packet, Raw
from scapy.layers.http import HTTP, HTTPRequest, HTTPResponse
from scapy.layers.inet import TCP, UDP
from scapy.layers.tls.all import TLS

from heifip.layers.ip import IPPacket
from heifip.plugins.header import CustomTCP, CustomUDP


class TransportPacket(IPPacket):
    def __init__(self, packet: Packet, address_mapping={}, layer_map={}):
        IPPacket.__init__(self, packet, address_mapping, layer_map)
        if TCP in self.layer_map:
            self.hash = hashlib.md5(f"{self.packet[TCP].flags},{self.packet[TCP].options}".encode('utf-8')).hexdigest()
            if TLS in self.layer_map:
                self.packet[TCP].remove_payload()
            if Raw in self.layer_map and not HTTP in self.layer_map:
                self.packet[TCP].remove_payload()
        elif UDP in self.layer_map:
            self.hash = hashlib.md5(f"{self.packet[UDP].name}".encode('utf-8')).hexdigest()
            if TLS in self.layer_map:
                self.packet[UDP].remove_payload()
            if Raw in self.layer_map and not HTTP in self.layer_map:
                self.packet[UDP].remove_payload()


    def header_preprocessing(self):
        if TCP in self.layer_map:
            layer_copy = self.packet[TCP]
            layer_copy = self.header_preprocessing_tcp(layer_copy)
            if self.packet[TCP].payload != None:
                layer_copy.payload = self.packet[TCP].payload
            self.packet[TCP] = layer_copy

        if UDP in self.layer_map:
            layer_copy = self.packet[UDP]
            layer_copy = self.header_preprocessing_udp(layer_copy)
            if self.packet[UDP].payload != None:
                layer_copy.payload = self.packet[UDP].payload
            self.packet[UDP] = layer_copy

        super().header_preprocessing()

    def header_preprocessing_tcp(self, layer_copy: Packet):
        return CustomTCP(flags=layer_copy.flags, options=layer_copy.options)

    def header_preprocessing_udp(self, layer_copy: Packet):
        return CustomUDP()



import os

import numpy as np
from PIL import Image as PILImage
from scapy.all import Packet

from heifip.exceptions import FIPWrongParameterException
from heifip.images import NetworkTrafficImage
from heifip.images.flow import FlowImage
from heifip.images.flow_tiled_auto import FlowImageTiledAuto
from heifip.images.flow_tiled_fixed import FlowImageTiledFixed
from heifip.images.markovchain import (MarkovTransitionMatrixFlow,
                                       MarkovTransitionMatrixPacket)
from heifip.images.packet import PacketImage
from heifip.layers import PacketProcessor, PacketProcessorType
from heifip.layers.packet import FIPPacket


class FIPExtractor:
    def __init__(self):
        self.processor = PacketProcessor()
        self.images_created = []
    
    def verify(self, image, min_image_dim: int, max_image_dim: int, remove_duplicates: bool):
        if image.shape[0] < min_image_dim or image.shape[1] < min_image_dim:
            return False

        if max_image_dim != 0 and (max_image_dim < image.shape[0] or max_image_dim < image.shape[1]):
            return False

        # if remove_duplicates:
        #     im_str = image.tobytes()
        #     if im_str in self.images_created:
        #         return False 
        #     else:
        #         self.images_created.append(im_str)

        return True

    def create_image_from_file(
            self,
            input_file: str,
            preprocessing_type: PacketProcessorType = PacketProcessorType.NONE,
            image_type: NetworkTrafficImage = PacketImage,
            min_image_dim: int = 0,
            max_image_dim: int = 0,
            min_packets_per_flow: int = 0,
            max_packets_per_flow: int = 0,
            remove_duplicates: bool = False,
            *args
        ):

        assert os.path.isfile(input_file)

        packets = self.processor.read_packets_file(input_file, preprocessing_type)

        images = self.__create_matrix(
            packets,
            preprocessing_type,
            image_type,
            min_image_dim,
            max_image_dim,
            min_packets_per_flow,
            max_packets_per_flow,
            remove_duplicates,
            *args
        )

        return images

    def create_image_from_packet(
            self,
            packets: [FIPPacket],
            preprocessing_type: PacketProcessorType = PacketProcessorType.NONE,
            image_type: NetworkTrafficImage = PacketImage,
            min_image_dim: int = 0,
            max_image_dim: int = 0,
            min_packets_per_flow: int = 0,
            max_packets_per_flow: int = 0,
            remove_duplicates: bool = False,
            *args
        ):

        packets = self.processor.read_packets_packet(packets, preprocessing_type)

        images = self.__create_matrix(
            packets,
            preprocessing_type,
            image_type,
            min_image_dim,
            max_image_dim,
            min_packets_per_flow,
            max_packets_per_flow,
            remove_duplicates,
            *args
        )

        return images
    
    def __create_matrix(
            self,
            packets: [FIPPacket],
            preprocessing_type: PacketProcessorType = PacketProcessorType.NONE,
            image_type: NetworkTrafficImage = PacketImage,
            min_image_dim: int = 0,
            max_image_dim: int = 0,
            min_packets_per_flow: int = 0,
            max_packets_per_flow: int = 0,
            remove_duplicates: bool = False,
            *args
        ):
        images = []
        if image_type == FlowImage:
            # when no file matches the preprocessing
            if len(packets) == 0 or len(packets) < min_packets_per_flow:
                return images

            # cut packets when too many are there
            if max_packets_per_flow != 0 and len(packets) > max_packets_per_flow:
                packets = packets[:max_packets_per_flow]

            image = FlowImage(packets, *args)
            if self.verify(image.matrix, min_image_dim, max_image_dim, remove_duplicates):
                images.append(image.matrix)

        elif image_type == FlowImageTiledFixed:
            # when no file matches the preprocessing
            if len(packets) == 0 or len(packets) < min_packets_per_flow:
                return images

            # cut packets when too many are there
            if max_packets_per_flow != 0 and len(packets) > max_packets_per_flow:
                packets = packets[:max_packets_per_flow]

            image = FlowImageTiledFixed(packets, *args)
            if self.verify(image.matrix, min_image_dim, max_image_dim, remove_duplicates):
                images.append(image.matrix)

        elif image_type == FlowImageTiledAuto:
            # when no file matches the preprocessing
            if len(packets) == 0 or len(packets) < min_packets_per_flow:
                return images

            # cut packets when too many are there
            if max_packets_per_flow != 0 and len(packets) > max_packets_per_flow:
                packets = packets[:max_packets_per_flow]

            image = FlowImageTiledAuto(packets, *args)
            if self.verify(image.matrix, min_image_dim, max_image_dim, remove_duplicates):
                images.append(image.matrix)

        elif image_type  == PacketImage:

            for packet in packets:
                image = PacketImage(packet, *args)
                if self.verify(image.matrix, min_image_dim, max_image_dim, remove_duplicates):
                    images.append(image.matrix)

        elif image_type == MarkovTransitionMatrixFlow:
            # when no file matches the preprocessing
            if len(packets) == 0 or len(packets) < min_packets_per_flow:
                return images

            # cut packets when too many are there
            if max_packets_per_flow != 0 and len(packets) > max_packets_per_flow:
                packets = packets[:max_packets_per_flow]

            image = MarkovTransitionMatrixFlow(packets, *args)
            if self.verify(image.matrix, min_image_dim, max_image_dim, remove_duplicates):
                images.append(image.matrix)

        elif image_type == MarkovTransitionMatrixPacket:
            for packet in packets:
                image = MarkovTransitionMatrixPacket(packet, *args)
                if self.verify(image.matrix, min_image_dim, max_image_dim, remove_duplicates):
                    images.append(image.matrix)
        else:
            raise FIPWrongParameterException

        return images

    def save_image(self, img, output_dir):
        pil_img = PILImage.fromarray(img)
        if not os.path.exists(os.path.realpath(os.path.dirname(output_dir))):
            try:
                os.makedirs(os.path.realpath(os.path.dirname(output_dir)))
            except:
                pass
        pil_img.save(f"{output_dir}_processed.png")

    def convert(self, img, target_type_min, target_type_max, target_type):
        imin = img.min()
        imax = img.max()

        a = (target_type_max - target_type_min) / (imax - imin)
        b = target_type_max - a * imax
        new_img = (a * img + b).astype(target_type)
        return new_img


import hashlib
from typing import Type

from scapy.all import Packet, RandIP, RandIP6, Raw
from scapy.layers.http import HTTP
from scapy.layers.inet import IP, TCP, UDP
from scapy.layers.inet6 import IPv6
from scapy.layers.tls.all import TLS

from heifip.layers.packet import EtherPacket
from heifip.plugins.header import CustomIP, CustomIPv6


class IPPacket(EtherPacket):
    def __init__(self, packet: Packet, address_mapping={}, layer_map={}):
        EtherPacket.__init__(self, packet, address_mapping, layer_map)
        if IP in self.layer_map:
            self.__filter_ipv4()
            self.hash = hashlib.md5(f"{self.packet[IP].version},{self.packet[IP].flags},{self.packet[IP].proto}".encode('utf-8')).hexdigest()
            if TLS in self.layer_map and not (TCP in self.layer_map or UDP in self.layer_map):
                self.packet[IP].remove_payload()
            if Raw in self.layer_map and not (TCP in self.layer_map or UDP in self.layer_map or HTTP in self.layer_map):
                self.packet[IP].remove_payload()
        elif IPv6 in self.layer_map:
            self.__filter_ipv6()
            self.hash = hashlib.md5(f"{self.packet[IPv6].version},{self.packet[IPv6].tc},{self.packet[IPv6].hlim}".encode('utf-8')).hexdigest()
            if TLS in self.layer_map and not (TCP in self.layer_map or UDP in self.layer_map):
                self.packet[IPv6].remove_payload()
            if Raw in self.layer_map and not (TCP in self.layer_map or UDP in self.layer_map or HTTP in self.layer_map):
                self.packet[IPv6].remove_payload()

    def __filter_ipv4(self):
        previous_src = self.packet[IP].src
        previous_dst = self.packet[IP].dst

        if previous_src in self.address_mapping:
            new_src = self.address_mapping[previous_src]
        else:
            new_src = RandIP()._fix()
            self.address_mapping[previous_src] = new_src

        if previous_dst in self.address_mapping:
            new_dst = self.address_mapping[previous_dst]
        else:
            new_dst = RandIP()._fix()
            self.address_mapping[previous_dst] = new_dst

        self.packet[IP].src = new_src
        self.packet[IP].dst = new_dst

    def header_preprocessing(self):        
        if IP in self.layer_map:
            layer_copy = self.packet[IP]
            layer_copy = self.header_preprocessing_ipv4(layer_copy)
            if self.packet[IP].payload != None:
                layer_copy.payload = self.packet[IP].payload
            self.packet[IP] = layer_copy
        if IPv6 in self.layer_map:
            layer_copy = self.packet[IPv6]
            layer_copy = self.header_preprocessing_ipv6(layer_copy)
            if self.packet[IPv6].payload != None:
                layer_copy.payload = self.packet[IPv6].payload
            self.packet[IPv6] = layer_copy

        super().header_preprocessing()

    def header_preprocessing_ipv4(self, layer_copy: Packet):
        return CustomIP(
            version=layer_copy.version,
            tos=layer_copy.tos,
            ttl=layer_copy.ttl,
            flags=layer_copy.flags,
            proto=layer_copy.proto,
        )

    def __filter_ipv6(self):
        previous_src = self.packet[IPv6].src
        previous_dst = self.packet[IPv6].dst

        if previous_src in self.address_mapping:
            new_src = self.address_mapping[previous_src]
        else:
            new_src = RandIP6()._fix()
            self.address_mapping[previous_src] = new_src

        if previous_dst in self.address_mapping:
            new_dst = self.address_mapping[previous_dst]
        else:
            new_dst = RandIP6()._fix()
            self.address_mapping[previous_dst] = new_dst

        self.packet[IPv6].src = new_src
        self.packet[IPv6].dst = new_dst

    def header_preprocessing_ipv6(self, layer_copy: Packet):
        return CustomIPv6(
            version=layer_copy.version,
            tc=layer_copy.tc,
            nh=layer_copy.nh,
            hlim=layer_copy.hlim,
        )



# File from pcap-splitter project
# Copyright (C) 2019 Santiago Hernandez Ramos <shramos@protonmail.com>
# For more information about the project: https://github.com/shramos/pcap-splitter

import subprocess


class PcapSplitter:
    """Split a .pcap file into different files."""

    def __init__(self, pcap_path, exefile_path="PcapSplitter"):
        # Checks if the PcapSplitter executable exists in path
        self._check_binary(exefile_path)
        self._exefile_path = exefile_path
        self._pcap_path = pcap_path

    def split_by_size(self, size_bytes, dest_path, pkts_bpf_filter=""):
        """Split files by size in bytes."""
        args = (self._exefile_path, "-f", self._pcap_path, "-o", dest_path,
                "-m", "file-size", "-p", str(size_bytes), "-i", pkts_bpf_filter)
        # Execute the PcapSplitter binary
        return self._execute(args).decode()

    def split_by_count(self, count_pkts, dest_path, pkts_bpf_filter=""):
        """Split files by packet count."""
        args = (self._exefile_path, "-f", self._pcap_path, "-o", dest_path,
                "-m", "packet-count", "-p", str(count_pkts), "-i", pkts_bpf_filter)
        # Execute the PcapSplitter binary
        return self._execute(args).decode()

    def split_by_client_ip(self, dest_path, pkts_bpf_filter=""):
        """split files by client IP, meaning all connections with the same client 
        IP will be in the same file."""
        args = (self._exefile_path, "-f", self._pcap_path, "-o", dest_path,
                "-m", "client-ip", "-i", pkts_bpf_filter)
        # Execute the PcapSplitter binary
        return self._execute(args).decode()

    def split_by_server_ip(self, dest_path, pkts_bpf_filter=""):
        """split files by server IP, meaning all connections with the same server 
        IP will be in the same file."""
        args = (self._exefile_path, "-f", self._pcap_path, "-o", dest_path,
                "-m", "server-ip", "-i", pkts_bpf_filter)
        # Execute the PcapSplitter binary
        return self._execute(args).decode()

    def split_by_server_port(self, dest_path, pkts_bpf_filter=""):
        """split files by IP src and dst (2-tuple), meaning all connections with 
        the same IPs will be in the same file."""
        args = (self._exefile_path, "-f", self._pcap_path, "-o", dest_path,
                "-m", "server-port", "-i", pkts_bpf_filter)
        # Execute the PcapSplitter binary
        return self._execute(args).decode()

    def split_by_ip_src_dst(self, dest_path, pkts_bpf_filter=""):
        """split files by IP src and dst (2-tuple), meaning all connections with 
        the same IPs will be in the same file."""
        args = (self._exefile_path, "-f", self._pcap_path, "-o", dest_path,
                "-m", "ip-src-dst", "-i", pkts_bpf_filter)
        # Execute the PcapSplitter binary
        return self._execute(args).decode()

    def split_by_session(self, dest_path, pkts_bpf_filter=""):
        """split files by connection (5-tuple), meaning all packets of a 
        connection will be in the same file."""
        args = (self._exefile_path, "-f", self._pcap_path, "-o", dest_path,
                "-m", "connection", "-i", pkts_bpf_filter)
        # Execute the PcapSplitter binary
        return self._execute(args).decode()

    def split_by_filter(self, bpf_filter, dest_path, pkts_bpf_filter=""):
        """split file into two files: one that contains all packets matching the 
        given BPF filter (file #0) and one that contains the rest of the packets 
        (file #1)."""
        args = (self._exefile_path, "-f", self._pcap_path, "-o", dest_path,
                "-m", "bpf-filter", "-p", bpf_filter, "-i", pkts_bpf_filter)
        # Execute the PcapSplitter binary
        return self._execute(args).decode()

    def split_by_round_robin(self, n_files, dest_path, pkts_bpf_filter=""):
        """split the file in a round-robin manner - each packet to a different 
        file."""
        args = (self._exefile_path, "-f", self._pcap_path, "-o", dest_path,
                "-m", "round-robin", "-p", str(n_files), "-i", pkts_bpf_filter)
        # Execute the PcapSplitter binary
        return self._execute(args).decode()

    def _execute(self, args):
        popen = subprocess.Popen(args, stdout=subprocess.PIPE)
        popen.wait()
        return popen.stdout.read()

    def _check_binary(self, exefile_path):
        try:
            subprocess.Popen(exefile_path, stdout=subprocess.PIPE)
        except FileNotFoundError:
            print("ERROR: PcapSplitter executable not found in the OS. Please \
            check that PcapPlusPlus is correctly installed and PcapSplitter \
            executable is in the path, or indicate the path of the PcapSplitter \
            executable by using the exefile_path parameter when instantiating \
            the PcapSplitter class.\n")